@@ -2647,6 +2647,10 @@ impl App {
             .unwrap_or_default()
     }
 
+    pub fn pwd_str(&self) -> String {
+        format!("{}\n", &self.pwd)
+    }
+
     pub fn selection_str(&self) -> String {
         self.selection
             .iter()
@@ -2689,6 +2693,7 @@ impl App {
             &builtin.create_file,
             &builtin.create_directory,
             &builtin.rename,
+            &builtin.duplicate_as,
             &builtin.delete,
             &builtin.sort,
             &builtin.filter,
