@@ -215,6 +215,9 @@ pub struct PanelUi {
 #[derive(Debug, Clone, Default, Serialize, Deserialize)]
 #[serde(deny_unknown_fields)]
 pub struct GeneralConfig {
+    #[serde(default)]
+    pub disable_debug_error_mode: bool,
+
     #[serde(default)]
     pub enable_mouse: bool,
 
@@ -520,6 +523,9 @@ pub struct BuiltinModesConfig {
     #[serde(default)]
     pub default: Mode,
 
+    #[serde(default)]
+    pub debug_error: Mode,
+
     #[serde(default)]
     pub recover: Mode,
 
@@ -544,6 +550,9 @@ pub struct BuiltinModesConfig {
     #[serde(default)]
     pub rename: Mode,
 
+    #[serde(default)]
+    pub duplicate_as: Mode,
+
     #[serde(default)]
     pub delete: Mode,
 
@@ -576,6 +585,7 @@ impl BuiltinModesConfig {
     pub fn get(&self, name: &str) -> Option<&Mode> {
         match name {
             "default" => Some(&self.default),
+            "debug_error" => Some(&self.debug_error),
             "recover" => Some(&self.recover),
             "selection ops" => Some(&self.selection_ops),
             "selection_ops" => Some(&self.selection_ops),
@@ -588,6 +598,8 @@ impl BuiltinModesConfig {
             "go to" => Some(&self.go_to),
             "go_to" => Some(&self.go_to),
             "rename" => Some(&self.rename),
+            "duplicate as" => Some(&self.duplicate_as),
+            "duplicate_as" => Some(&self.duplicate_as),
             "delete" => Some(&self.delete),
             "action" => Some(&self.action),
             "search" => Some(&self.search),
