@@ -14,6 +14,7 @@ use crossterm::event;
 use crossterm::execute;
 use crossterm::terminal as term;
 use mlua::LuaSerdeExt;
+use mlua::Value;
 use std::fs;
 use std::io::Write;
 use std::path::PathBuf;
@@ -32,15 +33,31 @@ pub fn get_tty() -> Result<fs::File> {
     }
 }
 
-fn call_lua(
+// In unix system, the std::env::current_dir() calls libc getcwd() that
+// returns physical path. As a workaround, this function tries to use `PWD`
+// environment variable that is configured by shell.
+fn get_current_dir() -> Result<PathBuf, std::io::Error> {
+    let cur = std::env::current_dir();
+    if let Ok(pwd) = std::env::var("PWD") {
+        if pwd.is_empty() {
+            cur
+        } else {
+            Ok(PathBuf::from(pwd))
+        }
+    } else {
+        cur
+    }
+}
+
+fn call_lua_heavy(
     app: &app::App,
     lua: &mlua::Lua,
     func: &str,
     _silent: bool,
 ) -> Result<Option<Vec<app::ExternalMsg>>> {
-    let arg = app.to_lua_arg();
-    let arg = lua.to_value(&arg)?;
-    lua::call_with_cache(lua, func, arg)
+    let arg = app.to_lua_ctx_heavy();
+    let arg = lua::serialize(lua, &arg)?;
+    lua::call(lua, func, arg)
 }
 
 fn call(app: &app::App, cmd: app::Command, silent: bool) -> Result<ExitStatus> {
@@ -109,25 +126,10 @@ pub struct Runner {
     extra_config_files: Vec<PathBuf>,
     on_load: Vec<app::ExternalMsg>,
     read_only: bool,
+    print_pwd_as_result: bool,
     selection: Vec<PathBuf>,
 }
 
-// In unix system, the std::env::current_dir() calls libc getcwd() that
-// returns physical path. As a workaround, this function tries to use `PWD`
-// environment variable that is configured by shell.
-fn get_current_dir() -> Result<PathBuf, std::io::Error> {
-    let cur = std::env::current_dir();
-    if let Ok(pwd) = std::env::var("PWD") {
-        if pwd.is_empty() {
-            cur
-        } else {
-            Ok(PathBuf::from(pwd))
-        }
-    } else {
-        cur
-    }
-}
-
 impl Runner {
     /// Create a new runner object passing the default arguments
     pub fn new() -> Result<Self> {
@@ -160,6 +162,7 @@ impl Runner {
             extra_config_files: cli.extra_config,
             on_load: cli.on_load,
             read_only: cli.read_only,
+            print_pwd_as_result: cli.print_pwd_as_result,
             selection: paths.collect(),
         })
     }
@@ -273,10 +276,6 @@ impl Runner {
                                 tx_msg_in.send(task)?;
                             }
 
-                            CacheDirectoryNodes(nodes) => {
-                                lua::cache_directory_nodes(&lua, &nodes)?;
-                            }
-
                             Quit => {
                                 result = Ok(None);
                                 break 'outer;
@@ -300,7 +299,12 @@ impl Runner {
                             }
 
                             PrintResultAndQuit => {
-                                result = Ok(Some(app.result_str()));
+                                result = if self.print_pwd_as_result {
+                                    Ok(Some(app.pwd_str()))
+                                } else {
+                                    Ok(Some(app.result_str()))
+                                };
+
                                 break 'outer;
                             }
 
@@ -455,7 +459,7 @@ impl Runner {
                             }
 
                             CallLuaSilently(func) => {
-                                match call_lua(&app, &lua, &func, false) {
+                                match call_lua_heavy(&app, &lua, &func, false) {
                                     Ok(Some(msgs)) => {
                                         app = app
                                             .handle_batch_external_msgs(msgs)?;
@@ -513,7 +517,7 @@ impl Runner {
                                 term::disable_raw_mode()?;
                                 terminal.show_cursor()?;
 
-                                match call_lua(&app, &lua, &func, false) {
+                                match call_lua_heavy(&app, &lua, &func, false) {
                                     Ok(Some(msgs)) => {
                                         app = app
                                             .handle_batch_external_msgs(msgs)?;
@@ -559,18 +563,55 @@ impl Runner {
                                 term::disable_raw_mode()?;
                                 terminal.show_cursor()?;
 
-                                let res: Result<Option<Vec<ExternalMsg>>> = lua
-                                    .load(&code)
-                                    .eval()
-                                    .and_then(|v| lua.from_value(v))
-                                    .map_err(Error::from);
+                                let res: Result<Value> =
+                                    lua.load(&code).eval().map_err(Error::from);
 
                                 match res {
-                                    Ok(Some(msgs)) => {
-                                        app = app
-                                            .handle_batch_external_msgs(msgs)?;
+                                    Ok(Value::Function(f)) => {
+                                        let arg = app.to_lua_ctx_heavy();
+                                        let res: Result<
+                                            Option<Vec<ExternalMsg>>,
+                                        > = lua
+                                            .to_value(&arg)
+                                            .and_then(|a| f.call(a))
+                                            .and_then(|v| lua.from_value(v))
+                                            .map_err(Error::from);
+                                        match res {
+                                            Ok(Some(msgs)) => {
+                                                app = app
+                                                    .handle_batch_external_msgs(
+                                                        msgs,
+                                                    )?;
+                                            }
+                                            Ok(None) => {}
+                                            Err(err) => {
+                                                app = app.log_error(
+                                                    err.to_string(),
+                                                )?;
+                                            }
+                                        }
+                                    }
+                                    Ok(v) => {
+                                        let res: Result<
+                                            Option<Vec<ExternalMsg>>,
+                                        > = lua
+                                            .from_value(v)
+                                            .map_err(Error::from);
+                                        match res {
+                                            Ok(Some(msgs)) => {
+                                                app = app
+                                                    .handle_batch_external_msgs(
+                                                        msgs,
+                                                    )?;
+                                            }
+                                            Ok(None) => {}
+                                            Err(err) => {
+                                                app = app.log_error(
+                                                    err.to_string(),
+                                                )?;
+                                            }
+                                        }
                                     }
-                                    Ok(None) => {}
                                     Err(err) => {
                                         app = app.log_error(err.to_string())?;
                                     }
@@ -598,18 +639,55 @@ impl Runner {
                             }
 
                             LuaEvalSilently(code) => {
-                                let res: Result<Option<Vec<ExternalMsg>>> = lua
-                                    .load(&code)
-                                    .eval()
-                                    .and_then(|v| lua.from_value(v))
-                                    .map_err(Error::from);
+                                let res: Result<Value> =
+                                    lua.load(&code).eval().map_err(Error::from);
 
                                 match res {
-                                    Ok(Some(msgs)) => {
-                                        app = app
-                                            .handle_batch_external_msgs(msgs)?;
+                                    Ok(Value::Function(f)) => {
+                                        let arg = app.to_lua_ctx_heavy();
+                                        let res: Result<
+                                            Option<Vec<ExternalMsg>>,
+                                        > = lua
+                                            .to_value(&arg)
+                                            .and_then(|a| f.call(a))
+                                            .and_then(|v| lua.from_value(v))
+                                            .map_err(Error::from);
+                                        match res {
+                                            Ok(Some(msgs)) => {
+                                                app = app
+                                                    .handle_batch_external_msgs(
+                                                        msgs,
+                                                    )?;
+                                            }
+                                            Ok(None) => {}
+                                            Err(err) => {
+                                                app = app.log_error(
+                                                    err.to_string(),
+                                                )?;
+                                            }
+                                        }
+                                    }
+                                    Ok(v) => {
+                                        let res: Result<
+                                            Option<Vec<ExternalMsg>>,
+                                        > = lua
+                                            .from_value(v)
+                                            .map_err(Error::from);
+                                        match res {
+                                            Ok(Some(msgs)) => {
+                                                app = app
+                                                    .handle_batch_external_msgs(
+                                                        msgs,
+                                                    )?;
+                                            }
+                                            Ok(None) => {}
+                                            Err(err) => {
+                                                app = app.log_error(
+                                                    err.to_string(),
+                                                )?;
+                                            }
+                                        }
                                     }
-                                    Ok(None) => {}
                                     Err(err) => {
                                         app = app.log_error(err.to_string())?;
                                     }
